--[[
	All rights are reserved unless explicitly stated below. The "license holder" is Minorin @ EU-Pozzo dell'Eternità (hideki87@gmail.com)
	
	# You may use this addon in accordance with all applicable terms set by Blizzard Entertainment for addon usage
	# You may modify this addon for personal use
	# You may NOT redistribute this addon in any way or form
--]]


-- register with Ace Libraries
local MB = LibStub("AceAddon-3.0"):NewAddon("Mercenary Buddy", "AceConsole-3.0", "AceEvent-3.0", "AceBucket-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("Mercenary Buddy")

local WasMercenary = false

local AddonVersion = "0.7 Beta"

local options = {
    type = "group",
    args = {
		description = {
			type = "description",
			name = L["Usage"],
			order = 0,
		},
        mkmacro = {
            type = "execute",
            name = L["Create Macros"],
			order = 1,
			func = function() MB:InitMacro() end,
        },
		--[[description = {
			type = "description",
			name = " ",
			order = 2,
		},--]]
		rmmacro = {
			type = "execute",
            name = L["Delete Macros"],
			order = 2,
			func = function() MB:DeleteMacro() end,
		},
    },
}

function MB:OnInitialize()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Mercenary Buddy", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Mercenary Buddy", "Mercenary Buddy")
	
	-- Register /racials to force update and /mercenary to print if mercenary spellID is found
	self:RegisterChatCommand("racials", "UpdateMacro", true)
	--self:RegisterChatCommand("mercenary", "IsMercenary")
	
	self:Print("version "..AddonVersion)
end

function MB:OnEnable()
	-- Register events that can make player a mercenary, Ashran and Battlegrounds
	self:RegisterEvent("BATTLEFIELD_MGR_ENTERED", "PreEnterBF")
	-- Delay further actions to complete ui load
	self:RegisterEvent("LOADING_SCREEN_DISABLED", "Loading")
end

function MB:Loading()
	self:UnregisterEvent("LOADING_SCREEN_DISABLED")
	self:RegisterEvent("PLAYER_ENTERING_BATTLEGROUND", "PreEnterBG") -- Register here to avoid being called when already in Battlegrounds
	local _,_,_,_,_,_,_,zone = GetInstanceInfo()
	if MB:IsMercenary() then
		if zone==1191 then -- Check if the player reloaded ui inside ashran
			--print("In Ashran")
			MB:EnterBF(true)
			return
		elseif UnitInBattleground("player") ~= nil then
			--print("In Battleground")
			MB:EnterBG(true)
			return
		end
	end
	-- Update racial macros at character login
	MB:UpdateMacro(true,false)

end

function MB:PreEnterBG()
	if UnitInBattleground("player") == nil then -- Check if called on reload ui inside a battleground
	--print("Pre Enter BG")
		--print("UnitInBattleground no")
		self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "EnterBG")
	end
end

function MB:EnterBG(forced)
	--print("Enter BG")
	if not forced then
		self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
		self:RegisterEvent("LOADING_SCREEN_DISABLED", "EnterBG", true)
		return
	end
	self:UnregisterEvent("LOADING_SCREEN_DISABLED")
	if MB:IsMercenary() then
		self:RegisterEvent("PLAYER_LEAVING_WORLD", "PreLeaveBG")
		WasMercenary = true
		MB:UpdateMacro(false,WasMercenary)
	else
		WasMercenary = false
	end
end

function MB:PreLeaveBG()
	--print("Pre Leave BG")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "LeaveBG")
end

function MB:LeaveBG()
	--print("Leave BG")
	self:UnregisterEvent("PLAYER_LEAVING_WORLD")
	--self:UnregisterBucket(BucketHandlerBG)
	--BucketHandlerBG = nil
	MB:UpdateMacro(false,WasMercenary)
end

function MB:PreEnterBF()
	local _,_,_,_,_,_,_,zone = GetInstanceInfo()
	if zone==1191 then -- Check if the player entered ashran as him/her faction and got insta-enter
		MB:EnterBF(true)
		return
	end
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "EnterBF" )
end

function MB:EnterBF(forced)
	--print("Enter BF")
	if not forced then
		self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
		self:RegisterEvent("LOADING_SCREEN_DISABLED", "EnterBF", true)
		return
	end
	self:UnregisterEvent("LOADING_SCREEN_DISABLED")
	--print("MB:IsMercenary() "..tostring(MB:IsMercenary()))
	if MB:IsMercenary() then
		self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "PreLeaveBF", 4)
		self:RegisterEvent("BATTLEFIELD_MGR_DROP_TIMER_STARTED", "PreLeaveBF", 2)
		self:RegisterEvent("BATTLEFIELD_MGR_EJECTED", "PreLeaveBF", 1)
		
		EventCountdown = 6 --Sum of events, when reach 0 in LeaveBF UpdateMacro() is called
		
		WasMercenary = true
		MB:UpdateMacro(false,WasMercenary)
	else
		WasMercenary = false
	end
end

function MB:ReEnterBF()
	self:UnregisterEvent("BATTLEFIELD_MGR_DROP_TIMER_CANCELED")
	MB:EnterBF(true)
end

function MB:PreLeaveBF(idEvent)
	--print("Leave BF WasMercenary "..tostring(WasMercenary))
	if idEvent == 4 then
		--print("UnregisterEvent(ZONE_CHANGED_NEW_AREA)")
		self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
		EventCountdown = EventCountdown - idEvent
	elseif idEvent == 2 then
		--print("UnregisterEvent(BATTLEFIELD_MGR_DROP_TIMER_STARTED)")
		self:UnregisterEvent("BATTLEFIELD_MGR_DROP_TIMER_STARTED")
		self:RegisterEvent("LANGUAGE_LIST_CHANGED", "LeaveBF")
		self:RegisterEvent("BATTLEFIELD_MGR_DROP_TIMER_CANCELED", "ReEnterBF")
		EventCountdown = EventCountdown - idEvent	
		MB:UpdateMacro(false,WasMercenary)
	elseif idEvent == 1 then
		--print("UnregisterEvent(BATTLEFIELD_MGR_EJECTED)")
		self:UnregisterEvent("BATTLEFIELD_MGR_EJECTED")
		self:UnregisterEvent("BATTLEFIELD_MGR_DROP_TIMER_CANCELED")
		self:UnregisterEvent("LANGUAGE_LIST_CHANGED")
		EventCountdown = EventCountdown - idEvent	
	end
end

function MB:LeaveBF()
	self:UnregisterEvent("LANGUAGE_LIST_CHANGED")
	MB:UpdateMacro(false,WasMercenary)
end

function MB:InitMacro()
	local macro = {}
	local c = 1
	for i = 1,3 do
		local index = GetMacroIndexByName("MercenaryBuddy"..i)
		--print(index)
		if index == 0 then
			local _ = CreateMacro("MercenaryBuddy".. i, "INV_MISC_QUESTIONMARK", " ")
			macro[c] = i
			c = c+1
			--print("MK Mercenary Buddy".. i)
		end
	end
	MB:UpdateMacro(true)
	local msg = L["Created Macro"]
	for key, value in ipairs(macro) do
		msg = msg.." MercenaryBuddy"..value
	end
	if c== 1 then
		self:Print(L["Macro already created"])
	else
		self:Print(msg)
	end
end

function MB:DeleteMacro()
	for i = 1,3 do
		local index = GetMacroIndexByName("MercenaryBuddy"..i)
		--print(index)
		if index > 0 then
			local _ = DeleteMacro(index)
			--print("RM Mercenary Buddy".. i)
		end
	end
	--MB:UpdateMacro()
	self:Print(L["Deleted Macro"])
end

function MB:LeavesCombat()
	--print("LeavesCombat")
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	MB:UpdateMacro()
end

function MB:IsMercenary()
	local mercenary=false
	for i=1,40 do 
		local _,_,_,_,_,_,_,_,_,_,spellID=UnitAura("player",i) 
		if spellID == 193864 or spellID == 193863 then 
			mercenary = true
			WasMercenary = true
		break 
		end 
	end
	--print("IsMercenary "..tostring(mercenary))
	return mercenary
end

function MB:UpdateMacro(forced,info)
	if not forced and not info then
		--print("not forced and was not mercenary")
		return
	end
	
	if InCombatLockdown() then
		self:RegisterEvent("PLAYER_REGEN_ENABLED", "LeavesCombat", forced, info)
		self:Print(L["You are in combat, Racials update delayed until out of combat"])
		return
	end
	
	if not forced then
		if info == false then
			return
		elseif info == nil then
			info = true
		end
	else 
		info = true
	end
	--print("info "..tostring(info))
	
	local name, rank, spellID
	local _, _, _, numSpells = GetSpellTabInfo(1)
	local racials = {}
	local c = 1
	
	for i=1,numSpells do
		name, rank, _, _, _, _, spellID = GetSpellInfo(i, "spell")
		if rank == "Racial" and GetSpellAvailableLevel(name) == 0 then
		--if rank == "Racial" then
			--print(GetSpellAvailableLevel(name))
			racials[c] = name
			c = c+1
			--print(name .." ".. rank.." "..spellID)
		end
	end
	
	local msg = L["New Racials"]
	
	for key, value in ipairs(racials) do
		local index = GetMacroIndexByName("MercenaryBuddy"..key)
		EditMacro(index, nil, nil, "#showtooltip\n/cast ".. value .."(Racial)")
		msg = msg.." "..key..":"..GetSpellLink(value)
		--print(key .." ".. value)
	end
	
	if info then
		self:Print(msg)
	end
end